# Part must be set for Xilinx bd script
set name ex
set part xc7a200tfbg484-2
set_part $part
set_param general.MaxThreads 4
set ILA true

# Set version and build time
if {[info exists ::env(CI_COMMIT_SHORT_SHA)]} {
    set revision [format {32'h%s} $::env(CI_COMMIT_SHORT_SHA)]
} else {
    set revision 0
}
set build_time [format {32'h%08x} [clock seconds]]
puts "REVISION: ${revision}"
puts "BUILD_TIME: ${build_time}"

# Get source directory from command line
set src_dir [lindex $argv 0]

# Override Xilinx message defaults
source [file nativename ${src_dir}/scripts/xil_msg.tcl]

# Read and generate block diagram sources
# source [file nativename ${src_dir}/src/bd_design.tcl]
# generate_target -force all [get_files .srcs/sources_1/bd/bd_design/bd_design.bd]

# Read and generate IP blocks
# set ips [list ]
# foreach ip $ips {
#     source [file nativename ${src_dir}/src/ip/${ip}.tcl]
#     synth_ip [get_ips $ip]
# }


# Read other source and constraint files
set file_list [glob -nocomplain ${src_dir}/src/*.vhd]
if {[llength $file_list]} {
    read_vhdl $file_list
}
set file_list [glob -nocomplain ${src_dir}/src/*.v]
if {[llength $file_list]} {
    read_verilog -sv $file_list
}
set file_list [glob -nocomplain ${src_dir}/src/*.sv]
if {[llength $file_list]} {
    read_verilog -sv $file_list
}
read_xdc ${src_dir}/constraints/${name}_physical.xdc
read_xdc ${src_dir}/constraints/${name}_timing.xdc


# Synthesis
synth_design -top ${name} -part $part -flatten_hierarchy none -generic REVISION=${revision} -generic BUILD_TIME=${build_time}

# write_ibis -force ${name}_post_synth.ibs
# write_csv -force ${name}_post_synth.csv
write_checkpoint -force ${name}_post_synth.dcp


# ILA
# Insert ILA here
if $ILA {
create_debug_core u_ila_0 ila
set_property C_DATA_DEPTH 1024 [get_debug_cores u_ila_0]
set_property C_TRIGIN_EN false [get_debug_cores u_ila_0]
set_property C_TRIGOUT_EN false [get_debug_cores u_ila_0]
set_property C_ADV_TRIGGER false [get_debug_cores u_ila_0]
set_property C_INPUT_PIPE_STAGES 0 [get_debug_cores u_ila_0]
set_property C_EN_STRG_QUAL false [get_debug_cores u_ila_0]
set_property ALL_PROBE_SAME_MU true [get_debug_cores u_ila_0]
set_property ALL_PROBE_SAME_MU_CNT 1 [get_debug_cores u_ila_0]
connect_debug_port u_ila_0/clk [get_nets [list clk_200_BUFG ]]
set_property port_width 10 [get_debug_ports u_ila_0/probe0]
set_property PROBE_TYPE DATA_AND_TRIGGER [get_debug_ports u_ila_0/probe0]
connect_debug_port u_ila_0/probe0 [get_nets [list {count[0]} {count[1]} {count[2]} {count[3]} {count[4]} {count[5]} {count[6]} {count[7]} {count[8]} {count[9]} ]]

}

# Optimize
opt_design
# write_checkpoint -force ${name}_post_opt_design.dcp


# ILA
if $ILA {
    write_debug_probes -force ${name}_ila.ltx
}

# Place
place_design


# Run optimization if timing violations exist after place
# if {[get_property SLACK [get_timing_paths -max_paths 1 -nworst 1 -setup]] < 0} {
#     puts "Found setup timing violations => running physical optimization"
#     phys_opt_design
# }


# Route
route_design
report_timing -file ${name}_timing_post_route.txt
report_timing_summary -file ${name}_timing_summary_post_route.txt
report_utilization -file ${name}_util_post_route.txt
report_utilization -hierarchical -file ${name}_util_post_route.txt -append
report_io -file ${name}_io_post_route.txt
report_clocks -file ${name}_clocks_post_route.txt
report_route_status -file ${name}_route_status.txt
report_power -file ${name}_power_post_route.txt
report_drc -file ${name}_drc_post_route.txt

write_checkpoint -force ${name}_post_route.dcp
# open_checkpoint ${name}
# write_hw_platform -fixed -include_bit -force -file ${name}.xsa


# Write bitstream
set_property BITSTREAM.CONFIG.CONFIGRATE 66 [current_design]
# set_property BITSTREAM.CONFIG.NEXT_CONFIG_ADDR 0x4000000 [current_design]
set_property BITSTREAM.CONFIG.SPI_32BIT_ADDR Yes [current_design]
set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]
set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]
set_property CONFIG_VOLTAGE 3.3 [current_design]
set_property CFGBVS VCCO [current_design]
set_property BITSTREAM.CONFIG.M1PIN PULLNONE [current_design]
set_property BITSTREAM.CONFIG.M2PIN PULLNONE [current_design]
set_property BITSTREAM.CONFIG.M0PIN PULLNONE [current_design]
set_property BITSTREAM.CONFIG.USR_ACCESS TIMESTAMP [current_design]
write_bitstream -force ${name}.bit

# Write memory and hardware definitions
# write_mem_info -force ${name}.mmi
# set_property platform.board_id                       TRM-TE0713     [current_project]
# set_property platform.name                           phoenix_proto  [current_project]
# set_property platform.design_intent.embedded         true           [current_project]
# set_property platform.design_intent.server_managed   false          [current_project]
# set_property platform.design_intent.external_host    false          [current_project]
# set_property platform.design_intent.datacenter       false          [current_project]
# write_checkpoint -force ${name}_post_route.dcp
# open_checkpoint ${name}_post_route.dcp
# write_hw_platform -force -include_bit ${name}.xsa
