proc duration int_time {
    set timeList [list]
    foreach div {86400 3600 60 1} mod {0 24 60 60} name {day hr min sec} {
        set n [expr {$int_time / $div}]
        if {$mod > 0} {set n [expr {$n % $mod}]}
        if {$n > 1} {
            lappend timeList "$n ${name}s"
        } elseif {$n == 1} {
            lappend timeList "$n $name"
        }
    }
    return [join $timeList]
}

if {$argc != 1} {
    return -code error "Usage: tclsh run.tcl project_name"
}

set build_script [lindex $argv 0]
puts "Building with - ${build_script}"
set build_script [file normalize ../scripts/${build_script}.tcl]
if ![file exists ${build_script}] {
    error "Build script does not exist" "${build_script}" 404
}

set vivado /Xilinx/Vivado/2020.2/bin/vivado
set vivado [file nativename $vivado]
set updatemem /Xilinx/Vivado/2020.2/bin/updatemem
set updatemem [file nativename $updatemem]

set bindir [file nativename ../bin]
set binsub [lindex $argv 0]
set bld_dir [file nativename ${bindir}/${binsub}]
set src_dir [file normalize ../]

# Bitfile generation
if ![file exists ${bindir}] {
    file mkdir ${bindir}
}
if {[file exists $bld_dir]} {
    file delete -force ${bld_dir}
}
file mkdir $bld_dir
cd $bld_dir


set tic [clock seconds]
puts "\n\n"
puts "********************************************************************************"
puts "* Calling Vivado with build file ${build_script}"
puts "********************************************************************************"
set status [exec $vivado -mode batch -source ${build_script} -tclargs ${src_dir} >@stdout 2>@stderr]
puts "********************************************************************************"
puts "* Finished Vivado build with file ${build_script}"
puts "********************************************************************************"
set toc [clock seconds]

puts "\n\nBuild took [duration [expr {$toc - $tic}]]."


# Comment "Bitfile generation" section above and uncomment below if inserting elf(s) into bit file only
# cd ${src_dir}/bin/${binsub}

# set status [exec $updatemem -meminfo ${name}.mmi -data ${src_dir}/src/boot.elf -proc i_boot_mb_wrapper/boot_mb_i/microblaze_0 -bit ${name}.bit -out ${name}.bit -force >@stdout 2>@stderr]
# puts $status

set status [exec $vivado -mode batch -source ${src_dir}/scripts/create_mcs.tcl -nolog -nojournal -notrace >@stdout 2>@stderr]
puts $status
