library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;


entity ex is
generic (
    REVISION        : std_logic_vector(31 downto 0) := (others=>'0');
    BUILD_TIME      : std_logic_vector(31 downto 0) := (others=>'0')
);    
port (
    CLK_200_P       : in    std_logic;
    CLK_200_N       : in    std_logic;
    EXPANSION       : out   std_logic_vector(9 downto 0)
);
end entity;


architecture rtl of ex is

    signal clk_200          : std_logic;
    signal rst_200          : std_logic;

    signal count            : std_logic_vector(9 downto 0);
    attribute keep : string;
    attribute keep of count : signal is "true";   

    constant ZERO           : std_logic := '0';
    constant ONE            : std_logic := '1';

begin

    i_ibugds_clk_200 : IBUFGDS port map (I=>CLK_200_P, IB=>CLK_200_N, O=>clk_200);

    i_rst_200 : SRLC32E
    generic map (
        INIT => X"FFFFFFFF")
    port map (
        Q       => rst_200, -- SRL data output
        Q31     => open, -- SRL cascade output pin
        A       => "11111", -- 5-bit shift depth select input
        CE      => ONE, -- Clock enable input
        CLK     => clk_200, -- Clock input
        D       => ZERO -- SRL data input
    );

    p_200: process(clk_200)
    begin
        if rising_edge(clk_200) then
            if rst_200 = '1' then
                count <= (others=>'0');
                EXPANSION <= (others=>'0');
            else
                count <= std_logic_vector(unsigned(count) + 1);
                EXPANSION <= count;
            end if;
        end if;
    end process;

end architecture;